@@ -12,6 +12,7 @@ import (
 
 	"google.golang.org/grpc"
 
+	"code.cloudfoundry.org/clock/fakeclock"
 	etcdraft "github.com/coreos/etcd/raft"
 	"github.com/coreos/etcd/raft/raftpb"
 	"github.com/docker/swarmkit/api"
@@ -22,7 +23,6 @@ import (
 	"github.com/docker/swarmkit/manager/state/raft"
 	"github.com/docker/swarmkit/manager/state/store"
 	"github.com/docker/swarmkit/testutils"
-	"github.com/pivotal-golang/clock/fakeclock"
 	"github.com/pkg/errors"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
@@ -60,13 +60,15 @@ func AdvanceTicks(clockSource *fakeclock.FakeClock, ticks int) {
 func WaitForCluster(t *testing.T, clockSource *fakeclock.FakeClock, nodes map[uint64]*TestNode) {
 	err := testutils.PollFunc(clockSource, func() error {
 		var prev *etcdraft.Status
+		var leadNode *TestNode
 	nodeLoop:
 		for _, n := range nodes {
 			if prev == nil {
 				prev = new(etcdraft.Status)
 				*prev = n.Status()
 				for _, n2 := range nodes {
-					if n2.Config.ID == prev.Lead && n2.ReadyForProposals() {
+					if n2.Config.ID == prev.Lead {
+						leadNode = n2
 						continue nodeLoop
 					}
 				}
@@ -84,7 +86,14 @@ func WaitForCluster(t *testing.T, clockSource *fakeclock.FakeClock, nodes map[ui
 			}
 			return errors.New("did not find leader in member list")
 		}
-		return nil
+		// Don't raise error just because test machine is running slowly
+		for i := 0; i < 5; i++ {
+			if leadNode.ReadyForProposals() {
+				return nil
+			}
+			time.Sleep(2 * time.Second)
+		}
+		return errors.New("leader is not ready")
 	})
 	require.NoError(t, err)
 }
